/*
 * Copyright (c) 2004 on behalf of the EU EGEE Project:
 * The European Organization for Nuclear Research (CERN),
 * Istituto Nazionale di Fisica Nucleare (INFN), Italy
 * Datamat Spa, Italy
 * Centre National de la Recherche Scientifique (CNRS), France
 * CS Systeme d'Information (CSSI), France
 * Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
 * Universiteit van Amsterdam (UvA), Netherlands
 * University of Helsinki (UH.HIP), Finland
 * University of Bergen (UiB), Norway
 * Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom
 *
 * Authors: Paolo Badino <Paolo.Badino@cern.ch>
 *
 * Version info: $Id: QUANTAnet_auth_strategy_c.hxx,v 1.5 2004/08/09 09:40:57 badino Exp $
 *
 * Revision history:
 * $ log$
 *
 */
 
#ifndef _QUANTAPLUS_QUANTAPLUS_AUTH_STRATEGY_CLASS
#define _QUANTAPLUS_QUANTAPLUS_AUTH_STRATEGY_CLASS

/**
 * QUANTAnet_auth_context_c
 * Base class for authorization Context
 */
class QUANTAnet_auth_context_c{

public:
    typedef enum {
        PM_NONE = 0,
        PM_INTEGRITY,
        PM_PRIVACY
    } ProtectionMode;

    /**
     * Destructor
     */
    virtual ~QUANTAnet_auth_context_c(){}
              
    /**
     * Return User's Name
     * @return User's Name or 0 if not available
     */
    virtual const char * getName() = 0;

    /**
     * Store User's Credentials in a file
     * @param filename [IN] the name of the file where to store the user 
     * credentials
     * @return 0 if success
     */
    virtual int exportCredentials(const char * filename) = 0;
   
	/**
     * Export User's Credentials to a buffer. Buffer Should be deallocated by 
     * calling QUANTAnet_auth_context_c::releaseBuffer
     * @param buffer [OUT] the buffer to store the user's credentials
     * @param buf_len [OUT] the buffer length
     * @return 0 if success
     */
    virtual int exportCredentials(char ** buffer, int * buf_len) = 0;
   
    /**
     * Check the if a Protection Mode is enabled
     * @param pm [IN] the protection mode to check
     * @return true if the requested protection service is available
     */
    virtual bool checkProtectionMode(ProtectionMode pm) = 0;
   
	/**
     * Encrypt Data according the the requested Protection Mode
     * @param src_buffer [IN] the buffer to encrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return 0 if success
     */
    virtual int encrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len, ProtectionMode pm = PM_PRIVACY) = 0;
	
    /**
     * Decrypt Data
     * @param src_buffer [IN] the buffer to decrypt
     * @param src_len [IN] the length of the source buffer
     * @param dst_buffer [OUT] the destination buffer. The buffer is allocated 
     *   internally and should be freed using the method @see releaseBuffer
     * @param dst_len [OUT] the length of the destination buffer
     * @return 0 if success
     */
    virtual int decrypt(const char * src_buffer, int src_len, char * &dst_buffer, int &dst_len) = 0;
    
    /**
     * Release a buffer previosly allocated by @see encrypt or @see descrypt
     * @param buffer [IN] the buffer to release
     */
    virtual void releaseBuffer(char * &buffer, int buffer_len) = 0;
  
protected:

    /**
     * Constructor
     */
    QUANTAnet_auth_context_c() {}  
};

/**
 * QUANTAnet_client_auth_strategy_c
 * Base class for Client authorization Context Factory
 */
class QUANTAnet_client_auth_strategy_c{

public:
    /**
     * Destructor
     */
    virtual ~QUANTAnet_client_auth_strategy_c(){}
    
    /**
     * Initialize
     * @return 0 if success
     */
    virtual int init() = 0;
    
    /**
     * Enable Encryption Service
     * @return 0 if success
     */
    virtual int enableEncryption() = 0;
    
    /**
     * Enable Delegation Service
     * @return 0 if success
     */
    virtual int enableDelegation() = 0;
    
    /**
     * Import the Credentials for Delegation from a proxy file
     * @param filename [IN] the file containing the credentials to be used
     * for delegation. This parameters is returned by a call to 
     * QUANTAnet_auth_context_c::exportCredentials
     * @return 0 if success
     */
    virtual int importCredentials(const char * filename) = 0;
    
    /**
     * Import the Credentials for Delegation from a buffer created by a call to
     * QUANTAnet_auth_context_c::exportCredentials
     * @param buffer [IN] the buffer containing the credentials to be used
     * for delegation
     * @param buf_len [IN] the length of the buffer
     * @return 0 if success
     */
    virtual int importCredentials(const char * buffer, int buf_len) = 0;
    
    /**
     * Establish the Authorization Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return the established Security Context
     */
    virtual QUANTAnet_auth_context_c * establish(int sock_handle) = 0;
    
protected:

    /**
     * Constructor
     */
    QUANTAnet_client_auth_strategy_c() {}    
};

/**
 * QUANTAnet_server_auth_strategy_c
 * Base class GSS for Server authorization Context Factory
 */
class QUANTAnet_server_auth_strategy_c{

public:
    /**
     * Destructor
     */
	virtual ~QUANTAnet_server_auth_strategy_c(){}
	
	/**
     * Initialize
     * @return 0 if success
     */
    virtual int init() = 0;
	
    /**
     * Accept a Security Context 
     * @param sock_handle [IN] handle of the socket used to contact the peer
     * @return the established Security Context
     */
    virtual QUANTAnet_auth_context_c * accept(int sock_handle) = 0;
   
    /**
     * Authorize an established Security Context 
     * @param ctx [IN] the security context to be authorized
     * @return 0 if success
     */
    virtual int authorize(QUANTAnet_auth_context_c * ctx) = 0;
    
protected:
    /**
     * Constructor
     */
    QUANTAnet_server_auth_strategy_c() {}   
    
};

#endif //_QUANTAPLUS_QUANTAPLUS_AUTH_STRATEGY_CLASS
